#Writen by Nicolas Mongiardino Koch during 2020 as supplementary material of
#Mongiardino Koch (2021) - Phylogenomic subsampling and the search for
#phylogenetically reliable loci. bioRxiv 2021.02.13.431075.
#https://doi.org/10.1101/2021.02.13.431075

#This iterates through the datasets and produces all files necessary to estimate
#site-wise rates of evolution using rate4site (Mayrose et al. 2004,
#https://www.tau.ac.il/~itaymay/cp/rate4site.html). This includes gene
#alignments, reference time-calibrated topology with matched taxa and a file to
#run on a computer cluster

library(ape)
library(phangorn)
library(phytools)
library(tidyverse)

#Change working directory to folder 'Datasets' containing all data files
setwd('')
rm(list=ls())
folders = substring(list.dirs(recursive = F), 2)

for(n in 1:length(folders)) {
  #read matrix, partition file and species tree
  data = read.phyDat(paste0(getwd(), folders[n], '/supermatrix_red_noOG.fa'), format = 'fasta', type = 'AA')
  data = as.AAbin(data)
  partitions = read.table(paste0(getwd(), folders[n], '/partitions_red_noOG.txt'), sep = ' ')
  partitions = as.tibble(partitions[,4])
  partitions = partitions %>% separate(value, into = c('Start', 'End'), sep = '-') %>% 
    mutate_if(is.character, as.numeric)
  tree = read.tree(paste0(getwd(), folders[n], '/dated_speciestree.tre'))
  
  #create a subfolder where to place the alignments
  dir.create(paste0(getwd(), folders[n], folders[n]))
  
  #loop through the loci
  for(i in 1:nrow(partitions)) {
    gene = as.character(data[,partitions$Start[i]:partitions$End[i]])
    
    #remove taxa that are represented by missing data
    deleted_taxa = which(apply(gene, 1, function(x) all(x == '-')))
    small_tree = tree
    
    if(length(deleted_taxa) > 0) {
      gene = gene[-deleted_taxa,]
      small_tree = drop.tip(small_tree, names(deleted_taxa)) 
    }
    
    #save alignment and pruned species tree on which to calculate rates
    write.phyDat(as.phyDat(as.AAbin(gene)), file = paste0(getwd(), folders[n], folders[n], '/gene', i, '.fa'), format = 'fasta')
    write.tree(small_tree, file = paste0(getwd(), folders[n], folders[n], '/tree', i, '.tre', sep = ''))
    
    #find the taxon with the least amount of missing data
    not_missing = apply(gene, 1, function(x) length(which(x != '-')))
    if(length(which(not_missing == max(not_missing))) > 1) {
      taxon = names(which(not_missing == max(not_missing))[1])
    } else {
      taxon = names(which(not_missing == max(not_missing)))
    }
    
    #open batch file with instructions
    if(i == 1) {
      line = paste0('#!/bin/bash', '\n', '#SBATCH --partition=week', '\n', '#SBATCH --job-name=', 
                    substring(folders[n], 2), '\n', '#SBATCH --ntasks=1', '\n', '#SBATCH --mem=5G', '\n', 
                    '#SBATCH --time=7-00:00:00', '\n', '#SBATCH --mail-user=nicolas.mongiardinokoch@yale.edu', '\n', 
                    '#SBATCH --mail-type=ALL', '\n', '\n')
      write(line, file = paste0(getwd(), folders[n], folders[n], '/run_rate4site.sh'), append = T)  
    }
    
    #add instructions specific to this gene, including calculating rates
    #relative to the most complete taxon
    line = paste0('rate4site -s ./gene', i, '.fa -t ./tree', i, '.tre -o site_rates', i, '.txt -a ', taxon, ' -bn')
    write(line, file = paste0(getwd(), folders[n], folders[n], '/run_rate4site.sh'), append = T)
  }
}
