#Writen by Nicolas Mongiardino Koch during 2020 as supplementary material of
#Mongiardino Koch (2021) - Phylogenomic subsampling and the search for
#phylogenetically reliable loci. bioRxiv 2021.02.13.431075.
#https://doi.org/10.1101/2021.02.13.431075

#This iterates through the datasets and reads the information rate4site (Mayrose
#et al. 2004, https://www.tau.ac.il/~itaymay/cp/rate4site.html) writes to
#standard output, including site rates in absolute values. This are first
#processed and saved into files named 'allrates_rate4site.csv' (which are
#provided as data files), and then a second step summarizes this information at
#the gene level, producing files names 'gene_properties.csv' (also provided).

library(MESS)
library(phangorn)
library(tidyverse)
library(psych)
library(PhyInformR)
library(splines)

#process rates-----------------------------------------------------------------------------------
#Change working directory to folder 'Datasets' containing all data files
setwd('')
rm(list=ls())
`%not in%` <- function (x, table) is.na(match(x, table, nomatch=NA_integer_))
folders = substring(list.dirs(recursive = F), 2)

for(n in 1:length(folders)) {
  #find output files and read partition files
  file = list.files(path = paste0(getwd(), folders[n]), pattern = 'slurm')
  lines = readLines(paste0(getwd(), folders[n], '/', file))
  partitions = read.table(paste0(getwd(), folders[n], '/partitions_red_noOG.txt'), sep = ' ')
  partitions = as_tibble(partitions[,4])
  partitions = partitions %>% separate(value, into = c('Start', 'End'), sep = '-') %>%
    mutate_if(is.character, as.numeric)

  runs = grep('START OF LOG FILE', lines)

  #get results for each individual gene
  for(i in 1:length(runs)) {
    if(i == length(runs)) {
      result = lines[runs[i]:length(lines)]
    } else {
      result = lines[runs[i]:(runs[i+1]-1)]
    }

    #if this loci failed, skip
    if(any(grepl('core', result))) next
    if(any(grepl('CANCELLED', result))) next

    #find which gene number this is
    gene = result[grep('tree file', result)]
    gene = gsub( '.tre', '', gsub('tree file is: ./tree', '', gene))
    if(!substring(gene, nchar(gene)) %in% c(0:9)) {
      gene = substr(gene, 1, (nchar(gene)-1))
    }
    gene = as.numeric(gene)

    #find positions with actual rates and extract only the numbers
    siterates = result[grep('rate of pos', result)]
    siterates = gsub(' rate of pos: ', '', siterates)
    siterates = as.numeric(unlist(lapply(strsplit(siterates, ' = '), '[', 2)))
    
    #add the rates and gene number to a growing dataset
    if(exists('all_rates')) {
      all_rates = rbind(all_rates, data.frame(gene = rep(gene, length(siterates)), rate = siterates))
    } else {
      all_rates = data.frame(gene = rep(gene, length(siterates)), rate = siterates)
    }
  
  }
  
  #add NA for genes that failes and notify which these are
  if(!all(1:nrow(partitions) %in% all_rates$gene)) {
    cat('Dataset', folders[n],  'genes', which(1:nrow(partitions) %not in% all_rates$gene), 'are missing', '\n')
    all_rates = rbind(all_rates, data.frame(gene = which(1:nrow(partitions) %not in% all_rates$gene),
                                            rate = rep(NA, length(which(1:nrow(partitions) %not in% all_rates$gene)))))
  }

  #save the results
  all_rates = all_rates[order(all_rates$gene),]
  write.csv(all_rates, file = paste0(getwd(), folders[n], '/allrates_rate4site.csv'), quote = F, row.names = F)
}

#Summarize the rates and calculate phylogenetic informativeness--------------------------------------------------------
#Remove rates >= 1, as these are failures in optimization
rm(list=ls())
`%not in%` <- function (x, table) is.na(match(x, table, nomatch=NA_integer_))
folders = substring(list.dirs(recursive = F), 2)

for(n in 1:length(folders)) {
  #read rates and remove those missing and those >=1
  rates = read.csv(paste0(getwd(), folders[n], '/allrates_rate4site.csv'))
  rates = rates[which(!is.na(rates[,2])),]
  rates = rates[which(rates[,2] < 1),]
  
  #read time-calibrated species tree
  tree = read.tree(paste0(getwd(), folders[n], '/dated_speciestree.tre'))
  
  #get genes with data
  genes = unique(rates[,1])
  
  #set-up a dataframe with rate and PI information
  summary_data = as.data.frame(matrix(0, nrow = length(genes), ncol = 5))
  colnames(summary_data) = c('gene', 'rate', 'harmonic_rate', 'iPI', 'iPIpen')
  summary_data[,1] = genes
  
  #loop through loci
  for(i in 1:nrow(summary_data)) {
    siterates = rates[which(rates[,1] == summary_data[i,1]),2]
    
    #use site rates to estimate informtiveness
    inform = PhyInformR:::inform.profile.generator(siterates, tree)
    
    #extract informativeness values at different points in time
    times = inform[1,]
    inform = inform[2,]
    
    #use interspline to smooth and remove phantom peaks close to the present
    yy = predict(interpSpline(times, inform))
    times = yy$x
    inform = yy$y
    
    #if this still did not remove the phantom peak, remove a couple of data
    #points close to the present
    if(inform[2] > inform[3] || inform[3] > inform[4]) {
      times = times[-c(2:4)]
      inform = inform[-c(2:4)]
      yy = predict(interpSpline(times, inform))
      times = yy$x
      inform = yy$y
    }
    
    #find peak in informativeness and penalize values at times older than the
    #peak by a factor proportional to their decay
    max = which(inform == max(inform))
    inform_pen = inform
    if(max != length(inform)) {
      for(j in (max + 1):length(inform)) {
        inform_pen[j] = inform[j]*(inform[j]/inform[max])
      }
    }
    
    #summarize using arithmetic and harmonic means (only used te latter), as
    #well as iPI and iPIpen (only used the latter as well)
    summary_data[i,2] = mean(siterates)
    summary_data[i,3] = harmonic.mean(siterates)
    summary_data[i,4] = MESS::auc(times, inform, type = 'spline')
    summary_data[i,5] = MESS::auc(times, inform_pen, type = 'spline')
  }
  
  #save the data
  write.csv(summary_data, file = paste0(getwd(), folders[n], '/rate_summary_pruned.csv'), quote = F, row.names = F)
}
