#Writen by Nicolas Mongiardino Koch during 2020 as supplementary material of
#Mongiardino Koch (2021) - Phylogenomic subsampling and the search for
#phylogenetically reliable loci. bioRxiv 2021.02.13.431075.
#https://doi.org/10.1101/2021.02.13.431075

#This iterates through the datasets, and produces 46 subsampled datasets,
#implementing 23 subsampling methods and 2 final dataset sizes. The matrices
#produced are available as files 'subsampling_datasets.zip'.

library(adephylo)
library(ape)
library(phangorn)
library(phytools)
library(tidyverse)

#Subsample-------------------------------------------------------------------------------------------------------
#Change working directory to folder 'Datasets' containing all data files
setwd('')
rm(list=ls())
`%not in%` <- function (x, table) is.na(match(x, table, nomatch=NA_integer_))
folders = substring(list.dirs(recursive = F), 2)

#set subsampling sizes and protocols
sizes = c(50, 250)
subsampling = data.frame(type = c('occupancy', 'length', 'treeness', 
                                  'bootstrap', 'robinson', 'variable_sites', 'clocklike', 'saturation', 'RCFV',
                                  'high_rate', 'mid_rate', 'low_rate', 'informativeness', 'PC1high', 'PC1low', 'PC2high', 'PC2low', 
                                  'randomA', 'randomB', 'randomC', 'randomD', 'randomE'), 
                         column = c(NA, 12, 14, 6, 8, 7, 2, 3, 4, 17, 17, 17, 19, 20, 20, 21, 21, NA, NA, NA, NA, NA), 
                         decreasing = c(NA, T, T, T, T, T, F, F, F, T, NA, F, T, T, F, T, F, NA, NA, NA, NA, NA))

for(n in 1:length(folders)) {
  #load matrix, partitions, properties and 
  data = read.phyDat(paste0(getwd(), folders[n], '/supermatrix_red.fa'), format = 'fasta', type = 'AA')
  data = as.AAbin(data)
  partitions = read.table(paste0(getwd(), folders[n], '/partitions_red.txt'), sep = ' ')
  partitions = as_tibble(partitions[,4])
  partitions = partitions %>% separate(value, into = c('Start', 'End'), sep = '-') %>%
    mutate_if(is.character, as.numeric)
  properties = read.csv(file = paste0(getwd(), folders[n], '/gene_properties.csv'))
  load(paste0(getwd(), folders[n], '/pca_geneproperties_nooutliers.Rda'))
  
  for(i in 1:nrow(subsampling)) {
    for(j in 1:length(sizes)) {
      #datasets are already sorted by occupancy so don't do anything
      if(subsampling[i,1] == 'occupancy') {
        properties_sorted = properties
      } else {
        if(is.na(subsampling[i,3])) {
          if(subsampling[i,1] == 'mid_rate') {
            properties_sorted = properties[order(abs(median(properties$harmonic_rate)-properties$harmonic_rate), decreasing = F),]
          } else {
            #random sorting
            properties_sorted = properties[sample(nrow(properties)),]
          }
        } else {
          #sort based on the protocols established in 'subsampling' dataframe
          properties_sorted = properties[order(properties[,subsampling[i,2]], decreasing = subsampling[i,3]),]
        }
      }
      
      #select the first 50 or 250
      selected_genes = properties_sorted[1:sizes[j],'genes']
      
      #sort the matrices
      positions = c()
      for(k in 1:length(selected_genes)) {
        positions = c(positions, partitions$Start[selected_genes[k]]:partitions$End[selected_genes[k]])
      }
      selected_data = data[,positions]
      
      #remove taxa that might be entirely missing
      all_missing = as.numeric(which(apply(as.character(selected_data), 1, function(x) all(x == '-'))))
      if(length(all_missing) > 0) {
        selected_data = selected_data[-all_missing,]
      }
      
      #create folder and file with IQTREE commands command
      if(n == 1 && i == 1 && j == 1) {
        dir.create('Subsampling_protocols')
        dir.create(paste0('./Subsampling_protocols/Results'))
        commands = '#!/bin/bash\n#SBATCH --partition=general\n#SBATCH --job-name=subsampling1\n#SBATCH --nodes=1\n#SBATCH --time=30-00:00:00\n#SBATCH --mail-user=nicolas.mongiardinokoch@yale.edu\n#SBATCH --mail-type=ALL\n#SBATCH -o subsampling1.out\n#SBATCH -e subsampling1.err\n#SBATCH --exclusive\n#SBATCH --constraint=avx2\n\nmodule load IQ-TREE/1.6.12-GCCcore-7.3.0-avx512\ncd /home/nm549/project/Subsampling_protocols\n'
        file = paste0('./Subsampling_protocols/', 'run_subsampling', n, '.sh')
        write(commands, file = file)
        for(k in 1:length(folders)) {
          dir.create(paste0('./Subsampling_protocols', folders[k]))
        }
      }
      
      #brake the runs into 4 separate files to run in parallel
      if(n == 6 || n == 11 || n == 16) {
        if(i == 1 && j == 1) {
          commands = paste0('#!/bin/bash\n#SBATCH --partition=general\n#SBATCH --job-name=subsampling', n, '\n#SBATCH --nodes=1\n#SBATCH --time=30-00:00:00\n#SBATCH --mail-user=nicolas.mongiardinokoch@yale.edu\n#SBATCH --mail-type=ALL\n#SBATCH -o subsampling', n, '.out\n#SBATCH -e subsampling', n, '.err\n#SBATCH --exclusive\n#SBATCH --constraint=avx2\n\nmodule load IQ-TREE/1.6.12-GCCcore-7.3.0-avx512\ncd /home/nm549/project/Subsampling_protocols\n')
          file = paste0('./Subsampling_protocols/', 'run_subsampling', n, '.sh')
          write(commands, file = file)
        }
      }
      
      #save datasets
      name = paste0(substring(folders[n], 2), '_', as.character(subsampling[i,1]), '_', sizes[j], 'genes.fa')
      write.phyDat(as.phyDat(selected_data), file = paste0('./Subsampling_protocols', folders[n], '/', name), format = 'fasta')
      
      #IQ-TREE commands
      commands = c(paste0('iqtree -s .', folders[n], '/', name, ' -st AA -m LG+F+G -bb 1000 -nt AUTO'), 
                   paste0('cp ' , folders[n], '/', '*contree ./Results'), 
                   paste0('rm *', gsub('.fa', '', name)))
      write(commands, file = file, append = T)
    }
  }
}
