#Writen by Nicolas Mongiardino Koch during 2020 as supplementary material of
#Mongiardino Koch (2021) - Phylogenomic subsampling and the search for
#phylogenetically reliable loci. bioRxiv 2021.02.13.431075.
#https://doi.org/10.1101/2021.02.13.431075

#This iterates through the datasets and calculates the values used to rank
#subsampling strategies (Robinson-Foulds similarity and avergare bootstrap). The
#results are saved to files 'subsampling_accuracy.Rda' which are provided.

library(adephylo)
library(ape)
library(dplyr)
library(phangorn)
library(phytools)
library(tidyverse)

#Change working directory to folder 'Datasets' containing all data files
setwd('')
rm(list=ls())
`%not in%` <- function (x, table) is.na(match(x, table, nomatch=NA_integer_))
folders = substring(list.dirs(recursive = F), 2)[1:18]

for(n in 1:length(folders)) {
  #open time-calibrated species tree
  truth = read.tree(paste0(getwd(), folders[n], '/dated_speciestree.tre'))
  
  #open all subsampled trees
  tree_files = list.files(path = paste0(getwd(), folders[n], '/subsampling_trees/'))
  tree_files = paste0(getwd(), folders[n], '/subsampling_trees/', tree_files)
  trees = lapply(tree_files, read.tree)
  for(i in 1:length(tree_files)) {
    trees[[i]] = drop.tip(trees[[i]], tip = which(trees[[i]]$tip.label %not in% truth$tip.label))
  }
  class(trees) = "multiPhylo"
  
  #mine information regarding strategy and sampling sizes from tree file names
  strategy = level = vector(length = length(tree_files))
  for(i in 1:length(tree_files)) {
    strategy[i] = strsplit(strsplit(tree_files[i], '/')[[1]][11], '_')[[1]][2]
    level[i] = parse_number(tail(strsplit(strsplit(tree_files[i], '/')[[1]][11], '_')[[1]], n = 1))
  }
  
  #gather all in dataset
  all_data = tibble(strategy, level) %>% mutate(clade = substring(folders[n], 2), RF_dist = NA, mean_BS = NA) %>%
    dplyr::select(clade, strategy, level, RF_dist, mean_BS)
  
  #calculate RF similarities and average BS support values
  for(i in 1:length(tree_files)) {
    this_tree = trees[[i]]
    this_truth = truth
    
    #match terminals present
    if(length(this_tree$tip.label) != length(this_truth$tip.label)) {
      this_truth = drop.tip(this_truth, tip = which(this_truth$tip.label %not in% this_tree$tip.label))
    }
    all_data$RF_dist[i] = 1 - RF.dist(this_truth, this_tree, normalize = TRUE, check.labels = TRUE)
    all_data$mean_BS[i] = mean(as.numeric(this_tree$node.label), na.rm = T)
  }
  
  #save results by datasets
  results = all_data %>% group_by(level) %>% group_split()
  save(results, file = paste0(getwd(), folders[n], '/subsampling_accuracy.Rda'))
}
