#Writen by Nicolas Mongiardino Koch during 2020 as supplementary material of
#Mongiardino Koch (2021) - Phylogenomic subsampling and the search for
#phylogenetically reliable loci. bioRxiv 2021.02.13.431075.
#https://doi.org/10.1101/2021.02.13.431075

#This code can replicate all of the analyses and graphs presented in the
#manuscript.

library(ape)
library(cluster)
library(colorspace)
library(corrplot)
library(dplyr)
library(factoextra)
library(ggplot2)
library(ggpmisc)
library(ggpubr)
library(MASS)
library(outliers)
library(phangorn)
library(phytools)
library(RColorBrewer)
library(readr)
library(tidyr)
library(vegan)

#Change working directory to folder 'Datasets' containing all data files
setwd('')
rm(list=ls())
`%not in%` <- function (x, table) is.na(match(x, table, nomatch=NA_integer_))
folders = substring(list.dirs(recursive = F), 2)[1:18]

#why remove outliers??-------------------------------------------------------------------------------------------------
#This reproduces Fig. 4 of the manuscript showing the benefits of eliminating
#outlier loci
n = 13
load(paste0(getwd(), folders[n], '/pca_geneproperties.Rda'))
properties = read.csv(file = paste0(getwd(), folders[n], '/gene_properties.csv'))
properties$PC_onebis = pca$scores[,1]
properties$PC_twobis = pca$scores[,2]
outliers = order(mahalanobis(pca$scores, rep(0, 7), cov(pca$scores)), decreasing = T)[1:(nrow(properties)/100)]
properties$outlier = ifelse(1:nrow(properties) %in% outliers, 'red', 'black')

ggplot(properties, aes(x = PC_onebis, y = PC_twobis)) + 
  geom_point(alpha = 0.3, aes(color = outlier), shape = 16, size = 3) + theme_bw() + 
  scale_color_manual(values = c('#BBCBD8','#E93658')) + theme(legend.position = "none") + theme(aspect.ratio=1)

tree = list.files(paste0(getwd(), folders[n], '/results/'), pattern = 'support.raxml.support')
plot(read.tree(paste0(getwd(), folders[n], '/results/', tree[which(parse_number(tree) == properties[outliers[1],'genes'])])), show.tip.label = F)
plot(read.tree(paste0(getwd(), folders[n], '/results/', tree[which(parse_number(tree) == properties[outliers[2],'genes'])])), show.tip.label = F)

properties[outliers[1],'outlier'] = 'yes'
ggplot(properties, aes(x = PC_onebis, y = PC_twobis)) + 
  geom_point(alpha = 0.3, aes(color = outlier), shape = 16, size = 3) + theme_bw() + 
  scale_color_manual(values = c('#BBCBD8','#E93658', 'black')) + theme(legend.position = "none") + theme(aspect.ratio=1)


n = 18
load(paste0(getwd(), folders[n], '/pca_geneproperties.Rda'))
properties = read.csv(file = paste0(getwd(), folders[n], '/gene_properties.csv'))
properties$PC_onebis = pca$scores[,1]
properties$PC_twobis = pca$scores[,2]
outliers = order(mahalanobis(pca$scores, rep(0, 7), cov(pca$scores)), decreasing = T)[1:(nrow(properties)/100)]
properties$outlier = ifelse(1:nrow(properties) %in% outliers, 'red', 'black')

ggplot(properties, aes(x = PC_onebis, y = PC_twobis)) + 
  geom_point(alpha = 0.3, aes(color = outlier), shape = 16, size = 3) + theme_bw() + 
  scale_color_manual(values = c('#BBCBD8','#E93658')) + theme(legend.position = "none") + theme(aspect.ratio=1)

tree = list.files(paste0(getwd(), folders[n], '/results/'), pattern = 'support.raxml.support')
plot(read.tree(paste0(getwd(), folders[n], '/results/', tree[which(parse_number(tree) == properties[outliers[1],'genes'])])), show.tip.label = F)
plot(read.tree(paste0(getwd(), folders[n], '/results/', tree[which(parse_number(tree) == properties[outliers[2],'genes'])])), show.tip.label = F)

properties[outliers[1],'outlier'] = 'yes'
ggplot(properties, aes(x = PC_onebis, y = PC_twobis)) + 
  geom_point(alpha = 0.3, aes(color = outlier), shape = 16, size = 3) + theme_bw() + 
  scale_color_manual(values = c('#BBCBD8','#E93658', 'black')) + theme(legend.position = "none") + theme(aspect.ratio=1)

#Correlations---------------------------------------------------------------------------------------------------------
#Code to replicate Figs S1 and S2 showing correlation across metrics
correlation_results = data.frame(corrs = rep(0, 36*18), prop = rep(rep(c('Average_BS', 'RF_distance', 'iPIpen'), each = 12), 18), 
           type = rep(NA, 36*18), clade = rep(substring(folders, 2), each = 36))
for(n in 1:length(folders)) {
  properties = read.csv(file = paste0(getwd(), folders[n], '/gene_properties.csv'))
  
  corrs = properties[,c(2,3,4,5,6,7,8,9,10,12,13,14,15,17,19)]
  if(n == 1) {
    pvals = ifelse(matrix(p.adjust(c(cor.mtest(
      corrs, conf.level = .95, method = 'spearman', exact = F)$p), method = 'hochberg'), byrow = F, nrow = 15, ncol = 15) < 0.05, 1, 0)
    
    corr = cor(corrs, method = 'spearman')/length(folders)
  } else {
    pvals = pvals + ifelse(matrix(p.adjust(c(cor.mtest(
      corrs, conf.level = .95, method = 'spearman', exact = F)$p), method = 'hochberg'), byrow = F, nrow = 15, ncol = 15) < 0.05, 1, 0)
    
    corr = corr + (cor(corrs, method = 'spearman')/length(folders))
  }
  correlation_results$corrs[which(correlation_results$clade == substring(folders[n], 2))] = c(cor(corrs, method = 'spearman')[(-c(5,7,15)),c(5,7,15)])
  correlation_results$type[which(correlation_results$clade == substring(folders[n], 2))] = rownames(cor(corrs, method = 'spearman')[(-c(5,7,15)),c(5,7,15)])
}

order = (correlation_results %>% group_by(type) %>% summarise(median = median(corrs)) %>% arrange(-median))$type

ggplot(correlation_results, aes(x = prop, y = corrs)) + geom_violin(aes(fill = prop), width = 0.5, alpha = 0.75) + 
  geom_dotplot(binaxis = 'y', stackdir = 'center', dotsize = 0.8, stackratio = 1.3) + geom_hline(yintercept = 0) + 
  scale_fill_manual(values = c('#FB5438', '#504D49', '#FFBA5D')) +
  facet_wrap(~factor(type, levels = order), nrow = 4, ncol = 3) + theme_bw()

correlation_results %>% mutate(corrs = abs(corrs)) %>% group_by(clade, prop) %>% arrange(clade, prop, desc(corrs)) %>% slice_max(1) %>% 
  ungroup() %>% dplyr::select(type) %>% group_by(type) %>% summarise(count = n())

corrplot(corr, method = 'ellipse', diag = F, outline = T, 
         type = 'upper', order = 'hclust', col = brewer.pal(n = 8, name = "RdBu"))

#Gather results from the PCA analysis of the datasets-----------------------------------------------------------------------------------------
get_pcs = c(1:2)
variance = data.frame(matrix(0, ncol = 8, nrow = 20))
for(n in 1:length(folders)) {
  load(file = paste0(getwd(), folders[n], '/pca_geneproperties_nooutliers.Rda'))
  variance[n,1:7] = pca2$sdev
  variance[n,8] = substring(folders[n], 2)
  for(i in 1:length(get_pcs)) {
    assign(paste0('pc_', get_pcs[i], '_', substring(folders[n], 2)), pca2$loadings[,get_pcs[i]])
  }
}

#extract proportions of variance explained by different PC axes
sdev = data.frame(t(apply(variance[,1:7], 1, function(x) (x^2)/(sum(x^2)))))
sdev = cbind(sdev, variance[,8])
colnames(sdev)[8] = 'Clade'
apply(sdev[,1:7], 2, mean)

#Invert all vectors ahead of clustering
nvar = length(pc_1_Actinopterygii)
pcs = ls(pattern = 'pc_')
for(i in 1:length(pcs)) {
  a = get(pcs[i])
  a = -a
  assign(paste0('inv_', pcs[i]), a)
}

pcs = ls(pattern = 'pc_')
all_pcs = matrix(0, ncol = length(get(pcs[1])), nrow = length(pcs))
colnames(all_pcs) = c(names(get(pcs[1])))
rownames(all_pcs) = pcs
for(i in 1:length(pcs)) {
  all_pcs[i,1:nvar] = get(pcs[i])
}

#perform hierarchical clustering and print only one half of the dendrogram
clusters = hclust(dist(all_pcs[,1:nvar], method = 'euclidean'), method = 'complete')
plot(clusters)
clusters_phylo = as.phylo(clusters)
clusters_phylo = drop.tip(clusters_phylo, unlist(Descendants(clusters_phylo, Children(clusters_phylo, length(clusters_phylo$tip.label)+1)[1], type = 'tips')))
plot(clusters_phylo)

#extract membership
clusterA_hierarchical = clusters_phylo$tip.label[unlist(Descendants(clusters_phylo, Children(clusters_phylo, length(clusters_phylo$tip.label)+1)[1]))]
clusterB_hierarchical = clusters_phylo$tip.label[unlist(Descendants(clusters_phylo, Children(clusters_phylo, length(clusters_phylo$tip.label)+1)[2]))]

#repeat with k means
K = kmeans(all_pcs, centers = 4, nstart = 10000)
fviz_cluster(K, data = all_pcs, geom = c("point", 'text'), stand = FALSE, frame.type = "convex")

some_pcs = all_pcs[-which(grepl('inv_', rownames(all_pcs))),]
some_K = kmeans(some_pcs, centers = 2, nstart = 10000)
fviz_cluster(some_K, data = some_pcs, geom = c("point", 'text'), stand = FALSE, frame.type = "convex")

#check that clustering is the same
shared_A = shared_B = rep(0,4)
for(i in 1:length(unique(K$cluster))) {
  this_cluster = names(which(K$cluster == i))
  shared_A[i] = length(which(this_cluster %in% clusterA_hierarchical))
  shared_B[i] = length(which(this_cluster %in% clusterB_hierarchical))
}

clusterA_Kmean = names(which(K$cluster == which(shared_A == max(shared_A))))
clusterB_Kmean = names(which(K$cluster == which(shared_B == max(shared_B))))

#do also partitioning around medoids
pam = pam(all_pcs, 4, metric = "euclidean", stand = F)
fviz_cluster(pam, data = all_pcs, geom = c("point", 'text'), stand = FALSE, frame.type = "convex")

#check that clustering is the same
shared_A = shared_B = rep(0,4)
for(i in 1:length(unique(pam$clustering))) {
  this_cluster = names(which(pam$clustering == i))
  shared_A[i] = length(which(this_cluster %in% clusterA_hierarchical))
  shared_B[i] = length(which(this_cluster %in% clusterB_hierarchical))
}

clusterA_pam = names(which(pam$clustering == which(shared_A == max(shared_A))))
clusterB_pam = names(which(pam$clustering == which(shared_B == max(shared_B))))

#define the two patterns as produced by identical clustering of the 3 methods
PatternA = clusterA_hierarchical[which(clusterA_hierarchical %in% clusterA_pam[which(clusterA_pam %in% clusterA_Kmean)])]
PatternB = clusterB_hierarchical[which(clusterB_hierarchical %in% clusterB_pam[which(clusterB_pam %in% clusterB_Kmean)])]

#which one is rate??------------------------------------------------------------------------------------------------
#correlate both PC axes against rate proxies
correlations = pvals = data.frame(matrix(0, nrow = 18, ncol = 6))
colnames(correlations) = colnames(pvals) = c('rate_PC1', 'rate_Pc2', 'tree_PC1', 'tree_PC2', 'clade', 'pattern')
for(n in 1:length(folders)) {
  load(paste0(getwd(), folders[n], '/pca_geneproperties_nooutliers.Rda'))
  properties = read.csv(file = paste0(getwd(), folders[n], '/gene_properties.csv'))
  properties = properties[complete.cases(properties),]
  
  correlations[n,1] = cor.test(log(properties$harmonic_rate), properties[,20], method = 'spearman')$estimate
  correlations[n,2] = cor.test(log(properties$harmonic_rate), properties[,21], method = 'spearman')$estimate
  correlations[n,3] = cor.test(log(properties$tree_rate), properties[,20], method = 'spearman')$estimate
  correlations[n,4] = cor.test(log(properties$tree_rate), properties[,21], method = 'spearman')$estimate
  correlations[n,5] = substring(folders[n], 2)
  corrs = abs(c(correlations[n,1], correlations[n,2]))
  if(max(corrs) > (2*min(corrs))) {
    correlations[n,6] = ifelse(correlations[n,1] > correlations[n,2], 'patternA', 'patternB')
  } else {
    correlations[n,6] = 'Ambiguous'
  }
  pvals[n,1] = cor.test(log(properties$harmonic_rate), properties[,20], method = 'spearman')$p.value
  pvals[n,2] = cor.test(log(properties$harmonic_rate), properties[,21], method = 'spearman')$p.value
  pvals[n,3] = cor.test(log(properties$tree_rate), properties[,20], method = 'spearman')$p.value
  pvals[n,4] = cor.test(log(properties$tree_rate), properties[,21], method = 'spearman')$p.value
}
correlations
colMeans(correlations[,1:4])

all_pcs_homolog = data.frame(all_pcs) %>% mutate(pc = rownames(all_pcs)) %>% filter(rownames(all_pcs) %in% c(PatternA, PatternB))
all_pcs_homolog = mutate(all_pcs_homolog, pattern = if_else(pc %in% PatternA, 'PatternB', 'PatternA'))

to_plot = pivot_longer(all_pcs_homolog, cols=colnames(all_pcs)[1:7], names_to = "Gene_property", values_to = "Eigenvector_values")
to_plot[which(to_plot$pattern == 'PatternB'),"Eigenvector_values"] = -to_plot[which(to_plot$pattern == 'PatternB'),"Eigenvector_values"]
to_plot = inner_join(to_plot, to_plot %>% group_by(pattern, Gene_property) %>% summarise(median_eigen = median(Eigenvector_values)))

#plot Fig. 1 showing distribution of eigenvectors
ggplot(to_plot, aes(x = Gene_property, y = Eigenvector_values)) + 
  geom_boxplot(aes(fill = median_eigen)) +  geom_hline(yintercept = 0) +
  theme_bw() + scale_fill_continuous_divergingx(palette = "Geyser") + coord_flip() + 
  facet_grid(rows = vars(factor(pattern))) + theme(legend.position = 'none')

#plot part of Fig. S6 showing eigenvectors for divergent datasets
ggplot(to_plot[which(grepl('Hexapoda', to_plot$pc)),], aes(x = Gene_property, y = Eigenvector_values)) + 
  geom_point(aes(color = Eigenvector_values), size = 2) +  geom_hline(yintercept = 0) +
  theme_bw() + scale_color_continuous_divergingx(palette = "Geyser") + coord_flip() + 
  facet_grid(rows = vars(factor(pattern))) + theme(legend.position = 'none')

ggplot(to_plot[which(grepl('Phasmatodea', to_plot$pc)),], aes(x = Gene_property, y = Eigenvector_values)) + 
  geom_point(aes(color = Eigenvector_values), size = 2) +  geom_hline(yintercept = 0) +
  theme_bw() + scale_color_continuous_divergingx(palette = "Geyser") + coord_flip() + 
  facet_grid(rows = vars(factor(pattern))) + theme(legend.position = 'none')

#extract eigenvalues (proportion of variance) to plot Fig. S8
eigenvalues = data.frame(matrix(0, ncol = 3, nrow = 16))
eigenvalues[,3] = correlations$clade[correlations$pattern != 'Ambiguous']
for(i in 1:nrow(eigenvalues)) {
  eigenvalues[i,1] = sdev[which(sdev$Clade == eigenvalues[i,3]),1]
  eigenvalues[i,2] = sdev[which(sdev$Clade == eigenvalues[i,3]),2]
}

colnames(eigenvalues) = c('PC1', 'PC2', 'Clade')
colMeans(eigenvalues[,1:2])
cor.test(eigenvalues[,1], eigenvalues[,2])

ggplot(eigenvalues, aes(x = PC1, y = PC2)) + geom_point() + geom_smooth(method = 'lm', se = F) + 
  geom_text(aes(label=Clade),hjust=0, vjust=0)

#Plot figs. 2 and S4 (i.e. correlation of PC 1 and PC2 against different rate
#proxies)
for(n in 1:length(folders)) {
  properties = read.csv(file = paste0(getwd(), folders[n], '/gene_properties.csv'))
  properties = properties[complete.cases(properties),]
  assign(paste0('properties_', substring(folders[n], 2)), properties)
  
  name = paste0('pcaAA', substring(folders[n], 2))
  assign(name, ggplot(get(paste0('properties_', substring(folders[n], 2))), aes(x = log(harmonic_rate), y = PC_one)) + 
           geom_point(alpha = 0.025, shape = 16) + geom_smooth(method = loess, se = F) + 
           xlab(expression(Rate~of~evolution~(log[10]))) + ylab('PCA 1') + theme_bw() +
           stat_cor(method = 'spearman', cor.coef.name = 'rho') + theme(aspect.ratio=1) + 
           theme(panel.grid.minor = element_blank()) + scale_y_continuous(labels = scales::number_format(accuracy = 1)) + 
           scale_x_continuous(labels = scales::number_format(accuracy = 1)))
  name = paste0('pcaAB', substring(folders[n], 2))
  assign(name, ggplot(properties, aes(x = log(harmonic_rate), y = -PC_two)) + 
           geom_point(alpha = 0.025, shape = 16) + geom_smooth(method = loess, se = F) + 
           xlab(expression(Rate~of~evolution~(log[10]))) + ylab('PCA 2') + theme_bw() +
           stat_cor(method = 'spearman', cor.coef.name = 'rho') + theme(aspect.ratio=1) + 
           theme(panel.grid.minor = element_blank()) + scale_y_continuous(labels = scales::number_format(accuracy = 1)) + 
           scale_x_continuous(labels = scales::number_format(accuracy = 1)))
  
  name = paste0('pcaBA', substring(folders[n], 2))
  assign(name, ggplot(properties, aes(x = log(tree_rate), y = PC_one)) + 
           geom_point(alpha = 0.025, shape = 16) + geom_smooth(method = loess, se = F) + 
           xlab(expression(Rate~of~evolution~(tree)~(log[10]))) + ylab('PCA 1') + theme_bw() +
           stat_cor(method = 'spearman', cor.coef.name = 'rho') + theme(aspect.ratio=1) + 
           theme(panel.grid.minor = element_blank()) + scale_y_continuous(labels = scales::number_format(accuracy = 1)) + 
           scale_x_continuous(labels = scales::number_format(accuracy = 1)))
  name = paste0('pcaBB', substring(folders[n], 2))
  assign(name, ggplot(properties, aes(x = log(tree_rate), y = -PC_two)) + 
           geom_point(alpha = 0.025, shape = 16) + geom_smooth(method = loess, se = F) + 
           xlab(expression(Rate~of~evolution~(tree)~(log[10]))) + ylab('PCA 2') + theme_bw() +
           stat_cor(method = 'spearman', cor.coef.name = 'rho') + theme(aspect.ratio=1) + 
           theme(panel.grid.minor = element_blank()) + scale_y_continuous(labels = scales::number_format(accuracy = 1)) + 
           scale_x_continuous(labels = scales::number_format(accuracy = 1)))
}

first_plot = c(ls(pattern = 'pcaAA'), ls(pattern = 'pcaAB'))
ggarrange(plotlist = lapply(as.list(first_plot), get), nrow = 4, ncol = 9)

second_plot = c(ls(pattern = 'pcaBA'), ls(pattern = 'pcaBB'))
ggarrange(plotlist = lapply(as.list(second_plot), get), nrow = 4, ncol = 9)

#Make Fig. S9 showing mean and variance in usefulness is positively correlated
#for genes evolving at similar rates. This will not employ the divergent
#datasets Hexapoda and Phasmatodea
folders2 = folders[-c(9,15)]
num_blocks = 25
for(n in 1:length(folders2)) {
  properties = read.csv(file = paste0(getwd(), folders2[n], '/gene_properties.csv'))
  properties = properties[complete.cases(properties),]
  
  rate = log10(properties$tree_rate)
  cuts = seq(min(rate), max(rate), length.out = (num_blocks + 1))
  
  block = rep(0, length(rate))
  for(i in 1:(length(cuts)-1)) {
    block[which(rate >= cuts[i] & rate < cuts[i+1])] = i
  }
  
  useful_mean = rep(NA, num_blocks)
  useful_var = rep(NA, num_blocks)
  midpoint_rate = rep(NA, num_blocks)
  num = rep(NA, num_blocks)
  
  for(i in 1:num_blocks) {
    useful_mean[i] = mean(properties$PC_two[which(block == i)])
    useful_var[i] = var(properties$PC_two[which(block == i)])
    midpoint_rate[i] = (cuts[i] + cuts[i+1])/2
    num[i] = length(properties$PC_two[which(block == i)])
  }
  
  data = data.frame(mean = useful_mean, var = useful_var, mid_rate = midpoint_rate, clade = rep(substring(folders2[n], 2), num_blocks), num = num)
  
  if(n == 1) {
    all_data = data
  } else {
    all_data = all_data %>% bind_rows(data)
  }
}

all_data2 = all_data %>% mutate(mean = -mean) %>% mutate(sd_plus = (mean+sqrt(var)), sd_minus = (mean-sqrt(var)))
all_data2 = all_data2[complete.cases(all_data2),]

#remove outliers automatically
for(i in 1:length(unique(all_data2$clade))) {
  this_dataset = all_data2[which(all_data2$clade == unique(all_data2$clade)[i]),]
  fit = lm(var ~ mean, data = this_dataset)
  residuals = resid(fit)
  scores(residuals)
  outlier = as.numeric(which(scores(residuals, type = 'chisq', prob = 1) > 0.95))
  if(length(outlier) > 0) {
    all_data2 = anti_join(all_data2, this_dataset[outlier,])
  }
}

ggplot(all_data2, aes(x = mean, y = var)) + 
  geom_point() + geom_smooth(method = 'lm') + facet_wrap(~clade, nrow = 2, scales = 'free') + theme_bw() +
  xlab('Mean usefulness') + ylab('Variance in usefulness')

#Report significant correlations
count = 0
for(i in 1:length(unique(all_data2$clade))) {
  this_data = all_data2[which(all_data2$clade == unique(all_data2$clade)[i]),]
  if(summary(lm(var ~ mean, data = this_data))[[4]][8] < 0.05) {
    cat(as.character(this_data$clade[1]), 'is significant!', '\n')
    count = count + 1
  } else {
    cat('Not', as.character(this_data$clade[1]), '\n')
  }
}
count

#Is it worth doing SortaDate?------------------------------------------------------------------------------------
#Make to make fig. S5
rm(list=ls())
`%not in%` <- function (x, table) is.na(match(x, table, nomatch=NA_integer_))
folders = substring(list.dirs(recursive = F), 2)
folders2 = folders[-c(9,15)]

dimensions_used = matrix(nrow = length(folders2), ncol = 7)
for(n in 1:length(folders2)) {
  properties = read.csv(file = paste0(getwd(), folders2[n], '/gene_properties.csv'))
  SortaDate = properties[order(-properties$robinson, properties$root_tip, -properties$tree_length, decreasing = F),]
  just_robinson = properties[order(-properties$robinson, decreasing = F),]
  dimensions_used[n,1:2] = c(length(which(SortaDate[1:50, 'genes'] %in% just_robinson [1:50, 'genes']))/50, 
                             length(which(SortaDate[1:300, 'genes'] %in% just_robinson [1:300, 'genes']))/300)
  SortaDate = properties[order(properties$root_tip, -properties$robinson, -properties$tree_length, decreasing = F),]
  just_roottip = properties[order(properties$root_tip, decreasing = F),]
  dimensions_used[n,3:4] = c(length(which(SortaDate[1:50, 'genes'] %in% just_roottip [1:50, 'genes']))/50, 
                             length(which(SortaDate[1:300, 'genes'] %in% just_roottip [1:300, 'genes']))/300)
  SortaDate = properties[order(-properties$tree_length, properties$root_tip, -properties$robinson, decreasing = F),]
  just_length = properties[order(-properties$tree_length, decreasing = F),]
  dimensions_used[n,5:7] = c(length(which(SortaDate[1:50, 'genes'] %in% just_length [1:50, 'genes']))/50, 
                             length(which(SortaDate[1:300, 'genes'] %in% just_length [1:300, 'genes']))/300, 
                             max(properties$ntax))
}

par(pty="s")
plot(dimensions_used[,1] ~ dimensions_used[,7])
plot(dimensions_used[,2] ~ dimensions_used[,7])

#2 decimals
dimensions_used = matrix(nrow = length(folders2), ncol = 7)
for(n in 1:length(folders2)) {
  properties = round(read.csv(file = paste0(getwd(), folders2[n], '/gene_properties.csv')), 2)
  SortaDate = properties[order(-properties$robinson, properties$root_tip, -properties$tree_length, decreasing = F),]
  just_robinson = properties[order(-properties$robinson, decreasing = F),]
  dimensions_used[n,1:2] = c(length(which(SortaDate[1:50, 'genes'] %in% just_robinson [1:50, 'genes']))/50, 
                             length(which(SortaDate[1:250, 'genes'] %in% just_robinson [1:250, 'genes']))/250)
  SortaDate = properties[order(properties$root_tip, -properties$robinson, -properties$tree_length, decreasing = F),]
  just_roottip = properties[order(properties$root_tip, decreasing = F),]
  dimensions_used[n,3:4] = c(length(which(SortaDate[1:50, 'genes'] %in% just_roottip [1:50, 'genes']))/50, 
                             length(which(SortaDate[1:250, 'genes'] %in% just_roottip [1:250, 'genes']))/250)
  SortaDate = properties[order(-properties$tree_length, properties$root_tip, -properties$robinson, decreasing = F),]
  just_length = properties[order(-properties$tree_length, decreasing = F),]
  dimensions_used[n,5:7] = c(length(which(SortaDate[1:50, 'genes'] %in% just_length [1:50, 'genes']))/50, 
                             length(which(SortaDate[1:250, 'genes'] %in% just_length [1:250, 'genes']))/250, 
                             max(properties$ntax))
}

par(pty="s")
plot(dimensions_used[,1] ~ dimensions_used[,7])
plot(dimensions_used[,2] ~ dimensions_used[,7])

#Explore subsamples--------------------------------------------------------------------------------------------------
rm(list=ls())
`%not in%` <- function (x, table) is.na(match(x, table, nomatch=NA_integer_))
folders = substring(list.dirs(recursive = F), 2)
folders2 = folders[-c(9,15)]

sizes = c(50, 250)
subsampling = data.frame(type = c('occupancy', 'length', 'treeness', 
                                  'bootstrap', 'robinson', 'variable', 'clocklike', 'saturation', 'RCFV',
                                  'high', 'mid', 'low', 'informativeness', 'PC1high', 'PC1low', 'PC2high', 'PC2low'), 
                         column = c(NA, 12, 14, 6, 8, 7, 2, 3, 4, 17, 17, 17, 19, 20, 20, 21, 21), 
                         decreasing = c(NA, T, T, T, T, T, F, F, F, T, NA, F, T, T, F, T, F))

distances_50 = data.frame(matrix(0, ncol = nrow(subsampling), nrow = nrow(subsampling)))
distances_250 = data.frame(matrix(0, ncol = nrow(subsampling), nrow = nrow(subsampling)))
rownames(distances_50) = rownames(distances_250) = colnames(distances_50) = colnames(distances_250) = subsampling$type

for(n in 1:length(folders2)) {
  properties = read.csv(file = paste0(getwd(), folders2[n], '/gene_properties.csv'))
  for(i in 1:nrow(subsampling)) {
    if(subsampling[i,1] == 'occupancy') {
      properties_sorted = properties
    } else {
      if(is.na(subsampling[i,3])) {
        if(subsampling[i,1] == 'mid_rate') {
          properties_sorted = properties[order(abs(median(properties$harmonic_rate)-properties$harmonic_rate), decreasing = F),]
        } else {
          properties_sorted = properties[sample(nrow(properties)),]
        }
      } else {
        properties_sorted = properties[order(properties[,subsampling[i,2]], decreasing = subsampling[i,3]),]
      }
    }
    
    assign(paste0('50genes_', rownames(distances_50)[i]), properties_sorted[1:sizes[1],'genes'])
    assign(paste0('250genes_', rownames(distances_250)[i]), properties_sorted[1:sizes[2],'genes'])
  }

  for(i in 1:(nrow(distances_50)-1)) {
    for(j in (i+1):nrow(distances_50)) {
      distances_50[i,j] = distances_50[i,j] + 1-(length(intersect(get(paste0('50genes_', rownames(distances_50)[i])), get(paste0('50genes_', rownames(distances_50)[j]))))/50)
      distances_250[i,j] = distances_250[i,j] + 1-(length(intersect(get(paste0('250genes_', rownames(distances_50)[i])), get(paste0('250genes_', rownames(distances_50)[j]))))/250)
    }
  }
}

distances_50 = distances_50/length(folders2)
distances_250 = distances_250/length(folders2)

distances_50[lower.tri(distances_50)] = t(distances_50)[lower.tri(distances_50)]
distances_250[lower.tri(distances_250)] = t(distances_250)[lower.tri(distances_250)]

pco = cmdscale(distances_250, k = 2, add = T)
plot(pco$points[,1], pco$points[,2])
text(pco$points[,1], pco$points[,2], rownames(distances_250))

nmds_250 = metaMDS(comm = distances_250, k = 2, try = 10000, autotransform = F)
stressplot(nmds_250)
sum((goodness(nmds_250))^2)
plot(nmds_250, display = 'sites')
text(nmds_250, display = 'sites')

nmds_50 = metaMDS(comm = distances_50, k = 2, try = 10000, autotransform = F)
stressplot(nmds_50)
sum((goodness(nmds_50))^2)
plot(nmds_50, display = 'sites')
text(nmds_50, display = 'sites')

#Load subsampling accuracy
for(n in 1:length(folders)) {
  #load results
  load(paste0(getwd(), folders[n], '/subsampling_accuracy.Rda'))
  
  #data for 50 loci
  this_50 = results[[1]]
  average = c(this_50$clade[1], 'average', as.numeric(this_50$level[1]), 
             as.numeric(mean(unlist(this_50[which(grepl('random', this_50$strategy)), "RF_dist"]))), 
             as.numeric(mean(unlist(this_50[which(grepl('random', this_50$strategy)), "mean_BS"]))))
  names(average) = colnames(this_50)
  
  this_50 = rbind(this_50, average)
  
  #replace multiple random replicates with their mean, and compute RF dist
  #relative to the average of random replicates. Rank methods
  this_50 = this_50 %>% mutate(RF_dist_stand = as.numeric(RF_dist) - as.numeric(this_50[which(this_50$strategy == 'average'), "RF_dist"])) %>% 
    mutate(mean_BS = as.numeric(mean_BS), RF_dist = as.numeric(RF_dist)) %>% filter(!grepl('random', strategy))
  this_50 = bind_cols(this_50[order(-this_50$RF_dist_stand, -as.numeric(this_50$mean_BS), decreasing = F),], rank = 1:18)
  this_50[which(this_50$strategy == 'average'), "strategy"] = 'random'
  
  #data for 250 loci
  this_250 = results[[2]]
  average = c(this_250$clade[1], 'average', as.numeric(this_250$level[1]), 
              as.numeric(mean(unlist(this_250[which(grepl('random', this_250$strategy)), "RF_dist"]))), 
              as.numeric(mean(unlist(this_250[which(grepl('random', this_250$strategy)), "mean_BS"]))))
  names(average) = colnames(this_250)
  
  #same processing for 250
  this_250 = rbind(this_250, average)
  this_250 = this_250 %>% mutate(RF_dist_stand = as.numeric(RF_dist) - as.numeric(this_250[which(this_250$strategy == 'average'), "RF_dist"])) %>% 
    mutate(mean_BS = as.numeric(mean_BS), RF_dist = as.numeric(RF_dist)) %>% filter(!grepl('random', strategy))
  this_250 = bind_cols(this_250[order(-this_250$RF_dist_stand, -as.numeric(this_250$mean_BS), decreasing = F),], rank = 1:18)
  this_250[which(this_250$strategy == 'average'), "strategy"] = 'random'
  
  #check if methods tied and modify ranks accordingly  
  if(any(duplicated(this_50[,5:6]))) {
    dupls = which(duplicated(this_50[,5:6]))
    for(i in 1:length(dupls)) {
      this_50$rank[dupls[i]:nrow(this_50)] = this_50$rank[dupls[i]:nrow(this_50)] - 1
    }
  }
  
  if(any(duplicated(this_250[,5:6]))) {
    dupls = which(duplicated(this_250[,5:6]))
    for(i in 1:length(dupls)) {
      this_250$rank[dupls[i]:nrow(this_250)] = this_250$rank[dupls[i]:nrow(this_250)] - 1
    }
  }
  
  #combine results for all datasets
  if(n == 1) {
    results_50 = this_50
    results_250 = this_250
  } else {
    results_50 = bind_rows(results_50, this_50)
    results_250 = bind_rows(results_250, this_250)
  }
}

#check results of deviating clades before removing them
results_50[which(results_50$clade == 'Hexapoda'),]
results_250[which(results_50$clade == 'Hexapoda'),]

results_50[which(results_50$clade == 'Phasmatodea'),]
results_250[which(results_50$clade == 'Phasmatodea'),]

hexa = rbind((results_50[which(results_50$clade %in% c('Hexapoda')),] %>% arrange(clade, strategy)), 
             (results_250[which(results_250$clade %in% c('Hexapoda')),] %>% arrange(clade, strategy))) %>% dplyr::select(strategy, level, rank) %>% 
  pivot_wider(names_from = level, values_from = rank)
phas = rbind((results_50[which(results_50$clade %in% c('Phasmatodea')),] %>% arrange(clade, strategy)), 
             (results_250[which(results_250$clade %in% c('Phasmatodea')),] %>% arrange(clade, strategy))) %>% dplyr::select(strategy, level, rank) %>% 
  pivot_wider(names_from = level, values_from = rank)

colnames(hexa) = colnames(phas) = c('strategy', 'genes50', 'genes250')

#plot the other half of Fig. S6 showing how strategies performed for Hexapoda
#and Phasmatodea
ggplot(hexa, aes(x = genes50, y = genes250)) + geom_point() + geom_text(aes(label = strategy))
ggplot(phas, aes(x = genes50, y = genes250)) + geom_point() + geom_text(aes(label = strategy))
                                                                        
results_50 = results_50[-which(results_50$clade %in% c('Hexapoda', 'Phasmatodea')),]
results_250 = results_250[-which(results_250$clade %in% c('Hexapoda', 'Phasmatodea')),]

#number of times a given method outranks random gene sampling
better_or_worse_50 = data.frame(matrix(0, ncol = 2, nrow = 17))
names(better_or_worse_50) = c('strategy', 'better')
better_or_worse_50$strategy = sort(unique(results_50$strategy))[-which(sort(unique(results_50$strategy)) == 'random')]

for(i in 1:length(unique(results_50$clade))) {
  this_clade = results_50 %>% filter(clade == unique(results_50$clade)[i])

  for(j in 1:nrow(better_or_worse_50)) {
    if(this_clade[which(this_clade$strategy == better_or_worse_50[j,1]), "rank"] < this_clade[which(this_clade$strategy == 'random'), "rank"]) {
      better_or_worse_50[j,2] = better_or_worse_50[j,2] + 1
    }
  }
}

better_or_worse_50$better = better_or_worse_50$better/length(folders2)

#same for 250 loci
better_or_worse_250 = data.frame(matrix(0, ncol = 2, nrow = 17))
names(better_or_worse_250) = c('strategy', 'better')
better_or_worse_250$strategy = sort(unique(results_250$strategy))[-which(sort(unique(results_250$strategy)) == 'random')]

for(i in 1:length(unique(results_250$clade))) {
  this_clade = results_250 %>% filter(clade == unique(results_250$clade)[i])
  
  for(j in 1:nrow(better_or_worse_250)) {
    if(this_clade[which(this_clade$strategy == better_or_worse_250[j,1]), "rank"] < this_clade[which(this_clade$strategy == 'random'), "rank"]) {
      better_or_worse_250[j,2] = better_or_worse_250[j,2] + 1
    }
  }
}

better_or_worse_250$better = better_or_worse_250$better/length(folders2)

order_50 = (results_50 %>% group_by(strategy) %>% summarize(rank = median(rank, .groups = 'drop')) %>% arrange(rank))$strategy
order_250 = (results_250 %>% group_by(strategy) %>% summarize(rank = median(rank, .groups = 'drop')) %>% arrange(rank))$strategy

color_50 = ifelse(better_or_worse_50[match(order_50, better_or_worse_50$strategy),2] > 0.5, 'red', 'white')
color_50[is.na(color_50)] = 'white'
color_250 = ifelse(better_or_worse_250[match(order_250, better_or_worse_250$strategy),2] > 0.5, 'red', 'white')
color_250[is.na(color_250)] = 'white'

#Plot Fig. 3A and S7 showing performance of subsampling methods
ggplot(results_50, aes(y = rank, x = factor(strategy, levels = order_50))) + 
  geom_boxplot(fill = color_50) + geom_text(data = better_or_worse_50, aes(x = strategy, y = 0, label = round(better, 2))) +
  theme_bw() + scale_x_discrete(guide=guide_axis(n.dodge=2))
ggplot(results_250, aes(y = rank, x = factor(strategy, levels = order_250))) + 
  geom_boxplot(fill = color_250) + geom_text(data = better_or_worse_250, aes(x = strategy, y = 0, label = round(better, 2))) +
  theme_bw() + scale_x_discrete(guide=guide_axis(n.dodge=2))

#Overlay accuracy on NMDS ordinatio plot and produce Fig. S3B
accuracy_250 = as.data.frame(results_250 %>% group_by(strategy) %>% summarize(RF_dist = mean(RF_dist), .groups = 'drop'))
rownames(accuracy_250) = accuracy_250$strategy
accuracy_250 = accuracy_250[-which(accuracy_250$strategy == 'random'),]
accuracy_250 = accuracy_250[match(rownames(scores(nmds_250)), rownames(accuracy_250)),]

plot(nmds_250, display = 'sites')
text(nmds_250, display = 'sites')
ordisurf(nmds_250, accuracy_250$RF_dist, add = T)

accuracy_50 = as.data.frame(results_50 %>% group_by(strategy) %>% summarize(RF_dist = mean(RF_dist), .groups = 'drop'))
rownames(accuracy_50) = accuracy_50$strategy
accuracy_50 = accuracy_50[-which(accuracy_50$strategy == 'random'),]
accuracy_50 = accuracy_50[match(rownames(scores(nmds_50)), rownames(accuracy_50)),]

plot(nmds_50, display = 'sites')
text(nmds_50, display = 'sites')
ordisurf(nmds_50, accuracy_50$RF_dist, add = T)