#Writen by Nicolas Mongiardino Koch during 2020 as supplementary material of
#Mongiardino Koch (2021) - Phylogenomic subsampling and the search for
#phylogenetically reliable loci. bioRxiv 2021.02.13.431075.
#https://doi.org/10.1101/2021.02.13.431075

#This iterates through the datasets, estimates all gene properties used in the
#manuscript, performs the PCA to infer rate and usefulness axes, and outputs
#results to files 'pca_geneproperties.Rda', 'pca_geneproperties.Rda' (after
#removing outliers) and 'gene_properties.csv', all of which are provided as data
#files.

library(adephylo)
library(ape)
library(phangorn)
library(phytools)
library(tidyverse)

#Change working directory to folder 'Datasets' containing all data files
setwd('')
rm(list=ls())
`%not in%` <- function (x, table) is.na(match(x, table, nomatch=NA_integer_))
folders = substring(list.dirs(recursive = F), 2)

#function to count invariant sites
inv = function(x) {
  pattern = unique(x)
  invariant = F
  if(length(pattern) == 1) {
    invariant = T
  }
  if(length(pattern) == 2 && '-' %in% pattern) {
    invariant = T
  }
  return(invariant)
}

#function to remove empty positions from the estimation of RCFV
remove_empty = function(x) {
  if('-' %in% names(unlist(x))) {
    missing = which(names(unlist(x)) == '-')
    x = x[-missing]
  }
  return(x)
}

for(n in 1:length(folders)) {
  #read matrix, parititons, species tree, gene trees and rate summary file
  data = read.phyDat(paste0(getwd(), folders[n], '/supermatrix_red_noOG.fa'), format = 'fasta', type = 'AA')
  data = as.AAbin(data)
  partitions = read.table(paste0(getwd(), folders[n], '/partitions_red_noOG.txt'), sep = ' ')
  partitions = as_tibble(partitions[,4])
  partitions = partitions %>% separate(value, into = c('Start', 'End'), sep = '-') %>%
    mutate_if(is.character, as.numeric)
  trees = read.tree(paste0(getwd(), folders[n], '/all_gene_trees.tre'))
  species_tree = read.tree(paste0(getwd(), folders[n], '/dated_speciestree.tre'))
  gene_rates = read.csv(paste0(getwd(), folders[n], '/rate_summary_pruned.csv'))
  
  #read missing gene trees txt file
  if('missing_gene_trees.txt' %in% list.files(paste0(getwd(), folders[n]))) {
    missing_trees = as.numeric(readLines(paste0(getwd(), folders[n], '/missing_gene_trees.txt')))
  } else {
    missing_trees = NULL
  }

  #get gene numbers that have either missing trees or missing rates
  if(nrow(gene_rates) != nrow(partitions)) {
    missing_rates = which(1:nrow(partitions) %not in% gene_rates[,1])
    tree_files = list.files(path = paste0(getwd(), folders[n], '/gene_trees'), pattern = 'raxml.support')
    tree_files = tree_files[order(parse_number(tree_files))]
    missing_rates_tree_pos = vector(length = length(missing_rates))
    for(i in 1:length(missing_rates)) {
      missing_rates_tree_pos[i] = which(grepl(paste0('gene', missing_rates[i], '_fa'), tree_files))
    }
  } else {
    missing_rates = NULL
  }
  
  #remove coresponding data so that rate, trees are partitions are perfectly
  #matched
  if(!is.null(missing_trees)) gene_rates = gene_rates[-which(gene_rates[,1] %in% missing_trees),]  
  if(!is.null(missing_rates)) trees = trees[-missing_rates_tree_pos]
  if(!is.null(missing_trees) || !is.null(missing_rates)) {
    partitions = partitions[-unique(c(missing_trees, missing_rates)),]
  }
  
  #read outgroups (these are present in dataset but missing from time-calibrated
  #species tree)
  conn = file(paste0(getwd(), folders[n], '/supermatrix_red.fa'), 'r')
  outgroups = scan(file = conn, what = "", sep = "\n", quiet = TRUE)[seq(1,1000,2)]
  close(conn)
  outgroups = outgroups[which(!is.na(outgroups))]
  outgroups = substring(outgroups, 2)
  outgroups = outgroups[which(outgroups %not in% species_tree$tip.label)]
  
  #initialize vectors for properties
  root_tip = saturation = missing = av.patristic = length = occupancy = variant.sites = RCFV =
    treeness = rate = average.support = robinson = ntax = tree_length = vector(length = length(trees))
  
  #loop through genes
  for(i in 1:length(trees)) {
    tree = trees[[i]]
    
    #reroot in either MRCA of ingroup or (if outgroups are missing) at midpoint
    #to estimate root-to-tip variance
    if(any(outgroups %in% tree$tip.label)) {
      tree = root(tree, node = getMRCA(tree, which(tree$tip.label %not in% outgroups)))
      root_tip[i] = sd(dist.nodes(tree)[getMRCA(tree, which(tree$tip.label %not in% outgroups)),which(tree$tip.label %not in% outgroups)])
      tree = drop.tip(trees[[i]], which(trees[[i]]$tip.label %in% outgroups))
    } else {
      root_tip[i] = sd(dist.nodes(midpoint.root(reroot(tree, (length(tree$tip.label)+5))))[(length(tree$tip.label)+1),(1:length(tree$tip.label))])
    }
    
    #RF similarity and BS support
    average.support[i] = mean(as.numeric(tree$node.label), na.rm = T)
    this_species_tree = drop.tip(species_tree, which(species_tree$tip.label %not in% tree$tip.label))
    robinson[i] = 1 - suppressMessages(RF.dist(this_species_tree, tree, normalize = TRUE, check.labels = TRUE))
    
    patristic = as.matrix(distTips(tree, tips = 'all', method = 'patristic', useC = T))
    
    #get alignent
    gene = as.character(data[,partitions$Start[i]:partitions$End[i]])
    total_tax = nrow(gene)
    
    #remove outgroups and entirely missing terminals
    if(any(rownames(gene) %not in% tree$tip.label)) {
      gene = gene[-which(rownames(gene) %not in% tree$tip.label),]
    }
    ntax[i] = nrow(gene)
    
    #remove empty positions generated by eliminated taxa
    all_missing = which(apply(gene, 2, function(x) all(x == '-')))
    if(length(all_missing) > 0) {
      gene = gene[,-all_missing]
    }
    
    variant.sites[i] = 1 - length(which(apply(gene, 2, inv)))/dim(gene)[2]
    missing[i] = length(which(gene == '-'))/(dim(gene)[1]*dim(gene)[2])
    length[i] = dim(gene)[2]
    occupancy[i] = dim(gene)[1]/total_tax
    
    #patristic distances and saturation
    p_dist = as.matrix(dist.hamming(as.phyDat(gene, type='AA'), ratio = TRUE, exclude = "pairwise"))
    p_dist = p_dist[order(colnames(p_dist)),order(colnames(p_dist))]
    patristic = patristic[order(colnames(patristic)),order(colnames(patristic))]
    p_dist = p_dist[lower.tri(p_dist)]
    patristic = patristic[lower.tri(patristic)]
    av.patristic[i] = mean(patristic)
    saturation[i] = 1 - lm(p_dist ~ patristic)$coefficients[[2]]
    if(saturation[i] < 0) saturation[i] = 0
    
    #RCFV
    mean_freqs = table(c(gene))
    states = sort(unique(c(gene)))
    if('-' %in% states) {
      mean_freqs = mean_freqs[-which(states == '-')]
      states = states[-which(states == '-')]
    }
    mean_freqs = mean_freqs/sum(mean_freqs)
    
    freqs = lapply(split(gene,seq(NROW(gene))),table)
    freqs = lapply(freqs, remove_empty)
    for(j in 1:length(freqs)) {
      if(!all(states %in% names(freqs[[j]]))) {
        miss = states[which(states %not in% names(freqs[[j]]))]
        add = rep(0, length(miss))
        names(add) = miss
        add = as.table(add)
        freqs[[j]] = as.table(c(freqs[[j]], add))
        freqs[[j]] = freqs[[j]][order(factor(names(freqs[[j]])))]
      }
    }
    
    freqs = lapply(freqs, function(x) x = x/sum(x))
    freqs = lapply(freqs, function(x) x = abs(x-mean_freqs))
    freqs = lapply(freqs, function(x) x = x/dim(gene)[1])
    freqs = lapply(freqs, function(x) sum(unlist(x)))
    RCFV[i] = sum(unlist(freqs))
    
    #treenes, length and tree rate
    treeness[i] = 1 - (sum(tree$edge.length[which(tree$edge[,2] %in% c(1:length(tree$tip.label)))])/sum(tree$edge.length))
    tree_length[i] = sum(tree$edge.length)
    rate[i] = tree_length[i]/length(tree$tip.label)
  }
  
  #get all data together (including rate and informativeness estimated previously)
  properties = data.frame(genes = gene_rates[,1], #gene number
                          root_tip, saturation, RCFV, av.patristic, #biases
                          average.support, robinson, #signal
                          variant.sites, #level of information
                          missing, occupancy, #occupancy
                          ntax, length, tree_length, treeness, #other
                          tree_rate = rate, mean_rate = gene_rates[,2], harmonic_rate = gene_rates[,3], #rates
                          iPI = gene_rates[,4], iPIpen = gene_rates[,5]) #informativeness

  #perform PCA and save results
  pca = princomp(properties[,2:8], cor = T, scores = T)
  save(pca, file = paste0(getwd(), folders[n], '/pca_geneproperties.Rda'))
  
  #detect outliers, remove (top 1%) and repeat
  outliers = order(mahalanobis(pca$scores, rep(0, 7), cov(pca$scores)), decreasing = T)[1:(nrow(properties)/100)]
  outliers = sort(outliers)
  pca2 = princomp(properties[-outliers,c(2:8)], cor = T, scores = T)
  save(pca2, file = paste0(getwd(), folders[n], '/pca_geneproperties_nooutliers.Rda'))
  
  scores1 = pca2$scores[,1]
  scores2 = pca2$scores[,2]
  for(i in 1:length(outliers)) {
    if(outliers[i] == 1) {
      scores1 = c(NA, scores1)
      scores2 = c(NA, scores2)
    } else {
      if(outliers[i] < length(scores1)) {
        scores1 = c(scores1[1:(outliers[i]-1)], NA, scores1[outliers[i]:length(scores1)])
        scores2 = c(scores2[1:(outliers[i]-1)], NA, scores2[outliers[i]:length(scores2)])
      } else {
        scores1 = c(scores1, NA)
        scores2 = c(scores2, NA)
      }
    }
  }
  
  #add to property matrix the scores of loci along PCs 1 and 2
  properties$PC_one = scores1
  properties$PC_two = scores2
  
  #save property data
  write.csv(properties, file = paste0(getwd(), folders[n], '/gene_properties.csv'), quote = F, row.names = F)
}
